@@ -1,16 +1,15 @@
-import igraph as ig
 import numpy as np
 from pathlib import Path
-from collections import deque
+from collections import deque, Counter
 import sys, random, time, pickle
 try:
-    from ctns.generator import generate_network, init_infection
+    from ctns.generator import generate_network, init_infection, compute_TR
     from ctns.steps import step
-    from ctns.utility import compute_TR, update_dump_report
+    from ctns.utility import update_dump_report
 except ImportError as e:
-    from generator import generate_network, init_infection
+    from generator import generate_network, init_infection, compute_TR
     from steps import step
-    from utility import compute_TR, update_dump_report
+    from utility import update_dump_report
 
 def run_simulation(n_of_families = 500,
     use_steps = True,
@@ -27,6 +26,7 @@ def run_simulation(n_of_families = 500,
     policy_test = "Random",
     contact_tracing_efficiency = 0.8,
     contact_tracing_duration = 14,
+    quarantine_efficiency = 0.4,
     use_random_seed = True,
     seed = 0,
     use_probabilities = False,
@@ -78,10 +78,16 @@ def run_simulation(n_of_families = 500,
         Number of avaiable tests
 
     policy_test: string
-        Strategy with which test are made. Can be Random, Degree Centrality, Betweenness Centrality
+        Strategy with which test are made. Can be Random, Degree Centrality, Betweenness Centrality or PBI
 
     contact_tracing_efficiency: float
-        The percentage of contacts successfully traced back in the past 14 days
+        The percentage of contacts successfully traced back in the past contact_tracing_duration days
+
+    contact_tracing_duration: int
+        How many days the contacts of a node are tracked back
+
+    quarantine_efficiency: float
+        The percentage of contacts put under quarantine
 
     use_random_seed: bool
         Id use or not a fixed random seed
@@ -152,12 +158,15 @@ def run_simulation(n_of_families = 500,
     if n_test < 0:
         print("Invalid number of test per day")
         sys.exit()
-    if not (policy_test == "Random" or policy_test == "Degree Centrality" or policy_test == "Betweenness Centrality"):
+    if not (policy_test == "Random" or policy_test == "Degree Centrality" or policy_test == "Betweenness Centrality" or "PBI"):
         print("Invalid test strategy")
         sys.exit()
     if contact_tracing_efficiency < 0 or contact_tracing_efficiency > 1:
         print("Invalid contact tracing efficiency")
         sys.exit()
+    if quarantine_efficiency < 0 or quarantine_efficiency > 1:
+        print("Invalid quarantine efficiency")
+        sys.exit()
     if contact_tracing_duration < 0:
         print("Invalid contatc tracing duration")
         sys.exit()
@@ -179,6 +188,9 @@ def run_simulation(n_of_families = 500,
     if dump_type != "full" and dump_type != "light":
         print("Invalid dump type")
         sys.exit()
+    if not use_probabilities and policy_test == "PBI":
+        print("Cannot use PBI if probability of being infected is not enabled")
+        sys.exit()
 
     # making parameters consistent
     if restriction_duration == 0 or social_distance_strictness == 0:
@@ -200,13 +212,12 @@ def run_simulation(n_of_families = 500,
         gamma = 0
         lambdaa = 0
 
-
     config = locals()
     a = time.perf_counter()
     # init network
     G = generate_network(n_of_families, use_probabilities)
     transmission_rate = compute_TR(G, R_0, infection_duration, incubation_days)
-    init_infection(G, n_initial_infected_nodes, use_probabilities)
+    init_infection(G, n_initial_infected_nodes)
 
     nets = deque(maxlen = contact_tracing_duration)
     if dump_type == "full":
@@ -231,7 +242,7 @@ def run_simulation(n_of_families = 500,
             net = step(G, sim_index, incubation_days, infection_duration, transmission_rate,
                              initial_day_restriction, restriction_duration, social_distance_strictness, 
                              restriction_decreasing, nets, n_test, policy_test, contact_tracing_efficiency,
-                             use_probabilities, gamma, lambdaa)
+                             quarantine_efficiency, use_probabilities, gamma, lambdaa)
             nets.append(net.copy())
             if dump_type == "full":
                 to_dump["nets"].append(net.copy())
@@ -245,17 +256,18 @@ def run_simulation(n_of_families = 500,
             net = step(G, sim_index, incubation_days, infection_duration, transmission_rate,
                              initial_day_restriction, restriction_duration, social_distance_strictness, 
                              restriction_decreasing, nets, n_test, policy_test, contact_tracing_efficiency,
-                             use_probabilities, gamma, lambdaa)
+                             quarantine_efficiency, use_probabilities, gamma, lambdaa)
             nets.append(net.copy())
             sim_index += 1
 
             if dump_type == "full":
-                to_dump.append(net.copy())
+                to_dump["nets"].append(net.copy())
             if dump_type == "light":
                 to_dump = update_dump_report(to_dump, net)
             
-            infected = to_dump["I"][-1]
-            exposed = to_dump["E"][-1]
+            network_report = Counter(net.vs["agent_status"])
+            infected = network_report["I"]
+            exposed = network_report["E"]
             if infected + exposed == 0:
                 break
 
@@ -267,27 +279,7 @@ def run_simulation(n_of_families = 500,
 
 def main():
 
-    n_of_families = None
-    use_steps = None
-    number_of_steps = None
-    incubation_days = None
-    infection_duration = None
-    initial_day_restriction = None
-    restriction_duration = None
-    social_distance_strictness = None
-    restriction_decreasing = None
-    n_initial_infected_nodes = None
-    R_0 = None
-    n_test = None
-    policy_test = None
-    contact_tracing_efficiency = None
-    contact_tracing_duration = None
-    use_random_seed = None
-    seed = None
     dump_type = None
-    use_probabilities = None
-    gamma = None
-    lambdaa = None
     path = None
 
     user_interaction = int(input("Press 0 to load the default values or 1 to manually input the configuration for the simulation: "))
@@ -306,9 +298,10 @@ def main():
         n_initial_infected_nodes = int(input("Please insert the number of initial infected individuals: "))
         R_0 = float(input("Please insert the value of R0: "))
         n_test = int(input("Please insert the number of available test per day: "))
-        policy_test = input("Please insert strategy with which test are made. Can be Random, Degree Centrality, Betweenness Centrality: ")
+        policy_test = input("Please insert strategy with which test are made. Can be Random, Degree Centrality, Betweenness Centrality or PBI (probability of being infected): ")
         contact_tracing_efficiency = float(input("Please insert a value between 0 and 1 to set the contact tracing efficiency: "))
         contact_tracing_duration = int(input("Please insert for how many days the contact tracing is computed: "))
+        quarantine_efficiency = float(input("Please insert a value between 0 and 1 to set the quarantine efficiency: "))
         use_random_seed = int(input("Press 1 use a fixed a random seed or 0 to pick a random seed: "))
         if use_random_seed:
             seed = int(input("Please insert the random seed: "))
@@ -322,7 +315,7 @@ def main():
         run_simulation(n_of_families, use_steps, number_of_steps, incubation_days, infection_duration,
             initial_day_restriction, restriction_duration, social_distance_strictness, restriction_decreasing,
             n_initial_infected_nodes, R_0, n_test, policy_test, contact_tracing_efficiency, contact_tracing_duration,
-            use_random_seed, seed, use_probabilities, gamma, lambdaa, dump_type, path)
+            quarantine_efficiency, use_random_seed, seed, use_probabilities, gamma, lambdaa, dump_type, path)
     else:
         dump_type = input("Please insert the dump type. Can be either full of light: ")
         path = input("Please insert the path with the file to dump. Please omit file type, that will be set automatically: ")
