@@ -1,7 +1,9 @@
-import igraph as ig
 import random
 import numpy as np
-from collections import Counter
+try:
+    from ctns.utility import retrive_to_test_quarantine, perform_test, compute_sd_reduction
+except ImportError as e:
+    from utility import retrive_to_test_quarantine, perform_test, compute_sd_reduction
 
 def generate_family_edges(G):
     """
@@ -200,6 +202,7 @@ def step_spread(G, incubation_days, infection_duration, transmission_rate, use_p
     None
     
     """
+
     if use_probabilities:
         old_prob =  G.vs["prob_inf"]
 
@@ -217,7 +220,8 @@ def step_spread(G, incubation_days, infection_duration, transmission_rate, use_p
                 if node["agent_status"] == "D":
                     node["quarantine"] = 0
                     node["test_result"] = - 1
-                    node["prob_inf"] = 0
+                    if use_probabilities:
+                        node["prob_inf"] = 0
                 
             # if it is still infective, spread the infection with his contacts
             if node["agent_status"] == "I":
@@ -274,7 +278,7 @@ def step_spread(G, incubation_days, infection_duration, transmission_rate, use_p
                 index = node.index
                 node["prob_inf"] = 1 - (1 - old_prob[index] * np.tanh(old_prob[index] + 0.5)) * nodes_contact_probs[index]
    
-def step_test(G, nets, incubation_days, n_new_test, policy_test, contact_tracing_efficiency, use_probabilities, lambdaa):
+def step_test(G, nets, incubation_days, n_new_test, policy_test, contact_tracing_efficiency, quarantine_efficiency, use_probabilities, lambdaa):
     """
     Test some nodes of the network and put the in quarantine if needed
     
@@ -294,11 +298,14 @@ def step_test(G, nets, incubation_days, n_new_test, policy_test, contact_tracing
 
     policy_test: string
         Test strategy
-        Can be ["Random, Degree Centrality, Betweenness Centrality"]
+        Can be ["Random, Degree Centrality, Betweenness Centrality, PBI"]
     
     contact_tracing_efficiency: float
         The percentage of contacts successfully traced
 
+    quarantine_efficiency: float
+        The percentage of contacts put under quarantine
+
     use_probabilities: bool
         Enables probabilities of being infected estimation
 
@@ -336,93 +343,75 @@ def step_test(G, nets, incubation_days, n_new_test, policy_test, contact_tracing
     high_priority_test_pool = [G.vs[i] for i in high_priority_test_pool]
     low_priority_test_pool = [G.vs[i] for i in low_priority_test_pool]
     for node in high_priority_test_pool:
-        if node["infected"]:
-            node["test_result"] = 1
-            node["quarantine"] = 14
-            node["test_validity"] = 14
-            node["prob_inf"] = 1
+        result = perform_test(node, incubation_days, use_probabilities)
+        if result:
             found_positive.add(node.index)
-        else:
-            node["test_result"] = 0
-            node["prob_inf"] = 0
-            node["test_validity"] = incubation_days
-
 
     to_test = list()
 
-    if policy_test == "Random" and n_new_test > 0:
-        to_test = random.sample(low_priority_test_pool, min(len(low_priority_test_pool), n_new_test))
+    if n_new_test:
+        if policy_test == "Random":
+            to_test = random.sample(low_priority_test_pool, min(len(low_priority_test_pool), n_new_test))
 
-    if policy_test == "Degree Centrality" and n_new_test > 0:
-        low_priority_test_pool_index = [x.index for x in low_priority_test_pool]
-        degree_results = G.strength(low_priority_test_pool_index, weights = "weight")
-        zipped_lists = zip(degree_results, low_priority_test_pool)
-        sorted_pairs = sorted(zipped_lists, reverse = True)
-        tuples = zip(*sorted_pairs)
-        _, sorted_nodes = [ list(tuple) for tuple in  tuples]
-        to_test = sorted_nodes[:min(n_new_test, len(low_priority_test_pool))]
+        if policy_test == "Degree Centrality":
+            low_priority_test_pool_index = [x.index for x in low_priority_test_pool]
+            degree_results = G.strength(low_priority_test_pool_index, weights = "weight")
+            to_test = retrive_to_test_quarantine(low_priority_test_pool, degree_results, n_new_test)
 
-    if policy_test == "Betweenness Centrality" and n_new_test > 0:
-        low_priority_test_pool_index = [x.index for x in low_priority_test_pool]
-        betweenness_results = G.betweenness(low_priority_test_pool_index, 
-                                            directed = False, weights = "weight",
-                                            cutoff = None)
-        zipped_lists = zip(betweenness_results, low_priority_test_pool)
-        sorted_pairs = sorted(zipped_lists, reverse = True)
-        tuples = zip(*sorted_pairs)
-        _, sorted_nodes = [ list(tuple) for tuple in  tuples]
-        to_test = sorted_nodes[:min(n_new_test, len(low_priority_test_pool))]
+        if policy_test == "Betweenness Centrality":
+            low_priority_test_pool_index = [x.index for x in low_priority_test_pool]
+            betweenness_results = G.betweenness(low_priority_test_pool_index, 
+                                                directed = False, weights = "weight",
+                                                cutoff = None)
+            to_test = retrive_to_test_quarantine(low_priority_test_pool, betweenness_results, n_new_test)
+
+        if policy_test == "PBI":
+            probs_infected = [i["prob_inf"] for i in low_priority_test_pool]
+            to_test = retrive_to_test_quarantine(low_priority_test_pool, probs_infected, n_new_test)
 
     for node in to_test:
-        if node["infected"]:
-            node["test_result"] = 1
-            node["quarantine"] = 14
-            node["test_validity"] = 14
-            node["prob_inf"] = 1
+        result = perform_test(node, incubation_days, use_probabilities)
+        if result:
             found_positive.add(node.index)
-        else:
-            node["test_result"] = 0
-            node["prob_inf"] = 0
-            node["test_validity"] = incubation_days
-    
-    # to_quarantine will contain family contacts (quarantine 100%), 
-    # possibly_quarantine will contain other contacts, quarantine influenced by contact tracing efficiency
+            
+    # tracked will contain family contacts (quarantine 100%), 
+    # possibly_tracked will contain other contacts, quarantine influenced by contact tracing efficiency
 
     if len(found_positive) > 0 and len(nets) > 0:
-        to_quarantine = set()
-        possibly_quarantine = set()
+        tracked = set()
+        possibly_tracked = set()
         # trace contacts
         for net in nets:
             for edge in net.es:
                 if edge["category"] == "family_contacts" \
                 and (edge.source in found_positive or edge.target in found_positive):
                     if edge.source in found_positive:
-                        to_quarantine.add(G.vs[edge.target].index)
+                        tracked.add(G.vs[edge.target].index)
                     else:
-                        to_quarantine.add(G.vs[edge.source].index)
+                        tracked.add(G.vs[edge.source].index)
                 else:
                     if edge.source in found_positive:
-                        possibly_quarantine.add(G.vs[edge.target].index)
+                        possibly_tracked.add(G.vs[edge.target].index)
                     if edge.target in found_positive:
-                        possibly_quarantine.add(G.vs[edge.source].index)
+                        possibly_tracked.add(G.vs[edge.source].index)
         
         # set diff to remove double contacts
-        possibly_quarantine = possibly_quarantine - to_quarantine
-        if len(possibly_quarantine) > int(len(possibly_quarantine) * contact_tracing_efficiency):
-            possibly_quarantine = random.sample(possibly_quarantine, int(len(possibly_quarantine) * contact_tracing_efficiency))
+        possibly_tracked = possibly_tracked - tracked
+        if len(possibly_tracked) > int(len(possibly_tracked) * contact_tracing_efficiency):
+            possibly_tracked = random.sample(possibly_tracked, int(len(possibly_tracked) * contact_tracing_efficiency))
         else:
-            possibly_quarantine = list(possibly_quarantine)
-        to_quarantine = list(to_quarantine) + possibly_quarantine
-
+            possibly_tracked = list(possibly_tracked)
+        tracked = list(tracked)
         if use_probabilities:
             # update prob of being infected of current contact 
-            for node in to_quarantine:
+            for node in tracked + possibly_tracked:
                 for contact in G.neighborhood(node)[1:]:
                     contact_node = G.vs[contact]
                     if contact_node["agent_status"] != "D" and not (contact_node["test_result"] == 0 and contact_node["agent_status"] == "R"):
                         current_contact_weight = G[node, contact]
                         contact_node["prob_inf"] = contact_node["prob_inf"] \
-                                                                        + lambdaa * np.e**(-(1 / current_contact_weight)) * (1 - contact_node["prob_inf"])
+                                                    + lambdaa * np.e**(-(1 / current_contact_weight)) * (1 - contact_node["prob_inf"])
+
             
             # update prob of being infected of past tracked contact 
                 for net_index in range(len(nets)):
@@ -430,13 +419,33 @@ def step_test(G, nets, incubation_days, n_new_test, policy_test, contact_tracing
                     for contact in net.neighborhood(node)[1:]:
                         contact_node = G.vs[contact]
                         if contact_node["agent_status"] != "D" and not (contact_node["test_result"] == 0 and contact_node["agent_status"] == "R"):
-                            if contact in to_quarantine:
+                            if contact in tracked + possibly_tracked:
                                 current_contact_weight = net[node, contact]
                                 contact_node["prob_inf"] = contact_node["prob_inf"] \
                                                             + lambdaa * np.e**(- (net_index + 1) * (1 / current_contact_weight)) * (1 - contact_node["prob_inf"])
 
-        to_quarantine = [G.vs[i] for i in to_quarantine]
+        
+        possibly_quarantine = list()
 
+        if quarantine_efficiency == 1:
+            possibly_quarantine = list(possibly_tracked)
+        elif quarantine_efficiency == 0 or possibly_tracked == list():
+            possibly_quarantine = list()
+        else:
+            if policy_test == "Random":
+                possibly_quarantine = random.sample(possibly_tracked, int(len(possibly_tracked) * quarantine_efficiency))
+            if policy_test == "Degree Centrality":
+                values = G.strength(possibly_tracked, weights = "weight")
+                possibly_quarantine = retrive_to_test_quarantine(possibly_tracked, values, int(len(possibly_tracked) * quarantine_efficiency))
+            if policy_test == "Betweenness Centrality":
+                values = G.betweenness(possibly_tracked, directed = False, weights = "weight", cutoff = None)
+                possibly_quarantine = retrive_to_test_quarantine(possibly_tracked, values, int(len(possibly_tracked) * quarantine_efficiency))
+            if policy_test == "PBI":
+                values = [G.vs[x]["prob_inf"] for x in possibly_tracked]
+                possibly_quarantine = retrive_to_test_quarantine(possibly_tracked, values, int(len(possibly_tracked) * quarantine_efficiency))
+
+        to_quarantine = [G.vs[i] for i in tracked + possibly_quarantine]
+        
         # put them in quarantine
         if to_quarantine != list() and to_quarantine != None:
             for node in to_quarantine:
@@ -445,7 +454,7 @@ def step_test(G, nets, incubation_days, n_new_test, policy_test, contact_tracing
 def step(G, step_index, incubation_days, infection_duration, transmission_rate,
          initial_day_restriction, restriction_duration, social_distance_strictness,
          restriction_decreasing, nets, n_test, policy_test, contact_tracing_efficiency,
-         use_probabilities, gamma, lambdaa):
+         quarantine_efficiency, use_probabilities, gamma, lambdaa):
     """
     Advance the simulation of one step
     
@@ -492,6 +501,9 @@ def step(G, step_index, incubation_days, infection_duration, transmission_rate,
     contact_tracing_efficiency: float
         The percentage of contacts successfully traced
 
+    quarantine_efficiency: float
+        The percentage of contacts put under quarantine
+
     use_probabilities: bool
         Enables probabilities of being infected estimation
 
@@ -528,64 +540,10 @@ def step(G, step_index, incubation_days, infection_duration, transmission_rate,
     step_spread(G, incubation_days, infection_duration, transmission_rate, use_probabilities, gamma)
           
     # make some test on nodes
-    step_test(G, nets, incubation_days, n_test, policy_test, contact_tracing_efficiency, use_probabilities, lambdaa)
+    step_test(G, nets, incubation_days, n_test, policy_test, contact_tracing_efficiency, quarantine_efficiency, use_probabilities, lambdaa)
 
     agent_status_report = list()
     for node in G.vs:
         agent_status_report.append(node["agent_status"])
 
-    return G  
-
-def compute_sd_reduction(step_index, initial_day_restriction, restriction_duration, social_distance_strictness):
-    """
-    Calculate the decreased social distance strictness index
-    
-    Parameters
-    ----------
-        
-    step_index : int 
-        index of the step
-    
-    initial_day_restriction: int
-        Day index from when social distancing measures are applied
-
-    restriction_duration: int
-        How many days the social distancing last. Use -1 to make the restriction last till the end of the simulation
-
-    social_distance_strictness: int
-        How strict from 0 to 4 the social distancing measures are. 
-        Represent the portion of contact that are dropped in the network (0, 25%, 50%, 75%, 100%)
-        Note that family contacts are not included in this reduction
-
-    Return
-    ------
-    social_distance_strictness: int
-        Updated value for social_distance_strictness
-        
-    """
-    
-    default_days = restriction_duration // social_distance_strictness
-    spare_days = restriction_duration - default_days * social_distance_strictness
-
-    strictness_sequence = list()
-
-    for i in range(social_distance_strictness):
-        updated_days = default_days
-        if spare_days > 0:
-            updated_days += 1
-            spare_days -= 1
-        strictness_sequence.append(updated_days)
-
-
-    for i in range(1, len(strictness_sequence)):
-      strictness_sequence[i] += strictness_sequence[i - 1]
-
-
-    social_distance_reduction = None
-
-    for i in range(len(strictness_sequence)):
-      if step_index - initial_day_restriction < strictness_sequence[i]:
-        social_distance_reduction = i
-        break
-
-    return social_distance_strictness - social_distance_reduction
\ No newline at end of file
+    return G  
\ No newline at end of file
